/*
 * @Author: your name
 * @Date: 2021-09-16 18:19:38
 * @LastEditTime: 2022-01-20 09:52:54
 * @LastEditors: Please set LastEditors
 * @Description: In User Settings Edi
 * @FilePath:
 * /NOP_EHR/media/igs/xnm/ros/RM/roadmodel/modules/common/road/boundary.h
 */
#pragma once

#include <stdint.h>

#include <list>

#include "boundary_segment.h"
#include "enum_list.h"
#include "glog/logging.h"
#include "lane_marker.h"
#include "line_geometry.h"
#include "polynom_line.h"

namespace rm {

/**
 * @class Boundary
 * @brief 用于生成corridor的boundary 可能是边线 也可能是road edge
 * 非输出的boundary
 */
class Boundary {
 public:
  /**
   * @brief: 存取函数
   * @param {LineGeometry&} geometry ： 存储的geometry
   * @return {*}
   */
  void set_geometry(const LineGeometry& geometry);

  const LineGeometry& geometry() const;
  /**
   * @brief: 存取函数
   * @param {LineType&} type ：存储的车道线类型
   * @return {*}
   */
  void set_line_type(const LineType& type);

  const LineType& line_type() const;
  /**
   * @brief: 存取函数
   * @param {LineColor&} color 存储的车道线颜色
   * @return {*}
   */
  void set_line_color(const LineColor& color);

  const LineColor& line_color() const;
  /**
   * @brief: 取函数
   * @param {*}
   * @return {*}
   */
  bool is_lane_marker() const;

  uint16_t sensor_source() const;

  bool is_front_marker() const;
  /**
   * @brief: 对车道线或road进行平移
   * @param {bool} is_left true:该线为左侧车道线 需向右平移
   * false：该线为右侧车道线 需向左平移
   * @return 平移车道线后的几何
   */
  PolynomLine ComputeShiftLine(bool is_left);
  /**
   * @brief: 根据分段的lane marker segment 构造一段lane marker
   * @param  lane_marker_segments 分段的lanemarker 需要构造一段boundary
   * @return {*}
   */
  void ComputeBoundaryFromLaneMarkers(
      const std::vector<LaneMarker>& lane_markers);

  void ComputeBackBoundaryFromLaneMarkers(
      const std::vector<LaneMarker>& lane_markers);

  /**
   * @brief: 友元函数 用于打印该类信息
   * @param {*}
   * @return {*}
   */
  friend std::ostream& operator<<(std::ostream& os, const Boundary& boundary);

  void set_valid(const bool valid);

  bool valid() const;

  std::vector<BoundarySegement>* mutable_boundary_segment_list();

  const std::vector<BoundarySegement>& boundary_segment_list() const;

 private:
  std::vector<BoundarySegement> boundary_segment_list_;
  LineGeometry geometry_;  //车道线表达
  LineType line_type_;     // 车道线类型
  LineColor line_color_;   //车道线颜色
  bool is_lane_marker_;    // true:该boundary为车道线 false:为road edge
                         // //TODO(ningmeng) 如果为roadedge 需要表达roadedge线型
  bool is_front_marker_;  // true: front boundary; false: back boundary
  uint16_t sensor_source_;
  bool valid_ = false;
  // RoadEdgeType road_edge_type_;
};

}  // namespace rm